package com.qmth.cqb.utils;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.stream.Stream;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;

/**
 * Created by songyue on 16/12/27.
 */
public final class CommonUtils {

    private static final String COMMON_PROPERTIES = "common.properties";

    private static final String[] CN_SMALL_NUM = { "零", "一", "二", "三", "四", "五", "六", "七", "八", "九" };

    private static final String[] CN_BIG_NUM = { "十", "百", "千", "万", "十", "百", "千", "亿", "十", "百", "千" };

    public static final String PAPER_TITLE="山东大学网络教育";
    public static final String PAPER_SUB_TITLE="2016-2017学年第一学期期末考试";

    /**
     * 加载properties配置文件
     * 
     * @param propertiesPath
     * @return
     */
    public static Properties loadProperties(String propertiesPath) {
        InputStream inputStream = null;
        Properties properties = new Properties();
        try {
            inputStream = ClassLoader.getSystemResourceAsStream(propertiesPath);
            BufferedReader bf = new BufferedReader(new InputStreamReader(inputStream));
            properties.load(bf);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return properties;
    }

    /**
     * 记载通用配置文件
     * 
     * @return
     */
    public static Properties loadCommonProperties() {
        return loadProperties(COMMON_PROPERTIES);
    }

    /**
     * 获取临时文件夹根路径
     * 
     * @return
     */
    public static String getTmpRootPath() {
        return (String) loadCommonProperties().get("tmp_root_path");
    }

    /**
     * 获取当前日期字符串
     * 
     * @return
     */
    public static String getCurDate() {
        return new SimpleDateFormat("yyyy-MM-dd").format(new Date());
    }

    /**
     * 获取当前日期时间字符串
     * 
     * @return
     */
    public static String getCurDateTime() {
        return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());
    }

    /**
     * 根据当前日期获取数字字符串
     * 
     * @return
     */
    public static String getCurNum() {
        return new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
    }

    /**
     * 根据枚举名称获取枚举
     * 
     * @param clazz
     * @param name
     * @param <T>
     * @return
     */
    public static <T> T getEnum(Class<T> clazz, String name) {
        T[] enumConstants = clazz.getEnumConstants();
        for (T _enum : enumConstants) {
            if (_enum.toString().equals(name)) {
                return _enum;
            }
        }
        return null;
    }

    /**
     * 校验题型
     * 
     * @param quesType
     * @return
     */
    public static boolean checkQuesType(String quesType) {
        return Stream.of(QuesStructType.values()).anyMatch(type -> type.getName().equals(quesType));
    }

    /**
     * 获取中文数字
     * 
     * @param inputNum
     * @return
     */
    public static String toCHNum(int inputNum) {
        String resultNum = "";
        String numStr = String.valueOf(inputNum);
        int len = numStr.length();
        for (int i = 0; i < len; i++) {
            int tmpNum = numStr.charAt(i) - '0';
            if (i != len - 1 && tmpNum != 0) {
                resultNum += CN_SMALL_NUM[tmpNum] + CN_BIG_NUM[len - 2 - i];
            } else {
                resultNum += CN_SMALL_NUM[tmpNum];
            }
        }
        return resultNum;

    }

    /**
     * iterator转List
     * 
     * @param <T>
     * @param iterator
     * @return
     */
    public static <T> List<T> toList(Iterable<T> iterator) {
        if (iterator == null) {
            throw new NullPointerException("Iterator must not be null");
        }
        final List<T> list = new ArrayList<T>();
        final Iterator<T> iter = iterator.iterator();
        while (iter.hasNext())
            list.add(iter.next());
        return list;
    }

    public static void main(String[] args) {
        // QuesStructType quesStructType = getEnum(QuesStructType.class,"单选");
        // System.out.println(quesStructType.getName());
        System.out.println(getCurNum());
    }
}
