package com.qmth.cqb.flow.model;

import java.io.Serializable;
import java.util.Date;

import org.activiti.engine.history.HistoricProcessInstance;
import org.activiti.engine.repository.Deployment;
import org.activiti.engine.repository.ProcessDefinition;
import org.activiti.engine.runtime.ProcessInstance;

/**
 * 
 * @Title: 流程定义模型
 * @Description: TODO
 * @author chenken
 * @date   2016年9月9日
 */
public class ProcessModel implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = -4693236062286086835L;
	
	/**
	 * 流程执行id
	 */
	private String processExecutionId;
	/**
	 * 流程定义id
	 */
	private String processDefinitionId;
	/**
	 * 流程实例id
	 */
	private String processInstanceId;
	
	/**
	 * 流程定义key
	 */
	private String processDefinitionKey;
	
	/**
	 * 流程定义名称
	 */
	private String processDefinitionName;
	/**
	 * 版本
	 */
	private Integer version;
	/**
	 * 流程发起人
	 */
	private String startUserId;
	/**
	 * 流程开始时间
	 */
	private Date startTime;
	
	/**
	 * 流程结束时间
	 */
	private Date endTime;
	/**
	 * 流程耗时
	 */
	private Long durationInMillis;
	/**
	 * 流程部署时间
	 */
	private Date deploymentTime;
	/**
	 * 流程部署ID
	 */
	private String deploymentId;
	/**
	 * 分类
	 */
	private String category;
	/**
	 * 资源名称
	 */
	private String resourceName;
	
	public ProcessModel(){}
	
	/**
	 * 组装运行时流程信息
	 * @param processInstance
	 * @param historicProcessInstance
	 */
	public ProcessModel(ProcessInstance processInstance,HistoricProcessInstance historicProcessInstance) {
		super();
		this.processExecutionId = processInstance.getId();
		this.processDefinitionId = processInstance.getProcessDefinitionId();
		this.processInstanceId = processInstance.getProcessInstanceId();
		this.processDefinitionName = processInstance.getProcessDefinitionName();
		this.startUserId = historicProcessInstance.getStartUserId();
		this.startTime = historicProcessInstance.getStartTime();
	}
	
	/**
	 * 组装流程信息
	 * @param processDefinition
	 * @param historicProcessInstance
	 */
	public ProcessModel(ProcessDefinition processDefinition,HistoricProcessInstance historicProcessInstance) {
		this.processDefinitionId = processDefinition.getId();
		this.processDefinitionName = processDefinition.getName();
		this.processDefinitionKey = processDefinition.getKey();
		this.category = processDefinition.getCategory();
		this.version = processDefinition.getVersion();
		this.processInstanceId = historicProcessInstance.getId();
		this.startUserId = historicProcessInstance.getStartUserId();
		this.startTime = historicProcessInstance.getStartTime();
		this.endTime = historicProcessInstance.getEndTime();
		this.durationInMillis = historicProcessInstance.getDurationInMillis();
	}
	
	/**
	 * 组装流程部署信息
	 * @param processDefinition
	 * @param deployment
	 */
	public ProcessModel(ProcessDefinition processDefinition,Deployment deployment) {
		this.processDefinitionId = processDefinition.getId();
		this.processDefinitionName = processDefinition.getName();
		this.processDefinitionKey = processDefinition.getKey();
		this.version = processDefinition.getVersion();
		this.category = processDefinition.getCategory();
		this.resourceName = processDefinition.getResourceName();
		this.deploymentTime = deployment.getDeploymentTime();
		this.deploymentId = deployment.getId();
	}

	public String getProcessExecutionId() {
		return processExecutionId;
	}

	public void setProcessExecutionId(String processExecutionId) {
		this.processExecutionId = processExecutionId;
	}

	public String getProcessDefinitionId() {
		return processDefinitionId;
	}

	public void setProcessDefinitionId(String processDefinitionId) {
		this.processDefinitionId = processDefinitionId;
	}

	public String getProcessInstanceId() {
		return processInstanceId;
	}

	public void setProcessInstanceId(String processInstanceId) {
		this.processInstanceId = processInstanceId;
	}

	public String getProcessDefinitionKey() {
		return processDefinitionKey;
	}

	public void setProcessDefinitionKey(String processDefinitionKey) {
		this.processDefinitionKey = processDefinitionKey;
	}

	public String getProcessDefinitionName() {
		return processDefinitionName;
	}

	public void setProcessDefinitionName(String processDefinitionName) {
		this.processDefinitionName = processDefinitionName;
	}

	public Integer getVersion() {
		return version;
	}

	public void setVersion(Integer version) {
		this.version = version;
	}

	public String getStartUserId() {
		return startUserId;
	}

	public void setStartUserId(String startUserId) {
		this.startUserId = startUserId;
	}

	public Date getStartTime() {
		return startTime;
	}

	public void setStartTime(Date startTime) {
		this.startTime = startTime;
	}

	public Date getEndTime() {
		return endTime;
	}

	public void setEndTime(Date endTime) {
		this.endTime = endTime;
	}

	public Long getDurationInMillis() {
		return durationInMillis;
	}

	public void setDurationInMillis(Long durationInMillis) {
		this.durationInMillis = durationInMillis;
	}

	public Date getDeploymentTime() {
		return deploymentTime;
	}

	public void setDeploymentTime(Date deploymentTime) {
		this.deploymentTime = deploymentTime;
	}

	public String getDeploymentId() {
		return deploymentId;
	}

	public void setDeploymentId(String deploymentId) {
		this.deploymentId = deploymentId;
	}

	public String getCategory() {
		return category;
	}

	public void setCategory(String category) {
		this.category = category;
	}

	public String getResourceName() {
		return resourceName;
	}

	public void setResourceName(String resourceName) {
		this.resourceName = resourceName;
	}
	
}
