/*
 * Copyright (c) 2016, FPX and/or its affiliates. All rights reserved.
 * Use, Copy is subject to authorized license.
*/
package com.qmth.cqb.flow.service;

import java.util.List;
import java.util.Map;

import org.activiti.engine.impl.task.TaskDefinition;
import org.activiti.engine.task.Task;

import com.qmth.cqb.flow.model.TaskModel;
import com.qmth.cqb.flow.model.WorkFlowModel;



/**
 * @Title: ComTaskService
 * @Description:TODO
 * @Company:4px
 * @author chenken
 * @date 2016年6月24日
 * @since v1.0.0
 */
public interface ComTaskService {
	/**
	* 查询个人可以办理的任务
	* assignee	任务办理人		必须
	* category	系统类别			非必须
	* @param assignee 用户ID
	* @return  
	* @throws
	 */
	public List<TaskModel> findPersonalTaskList(String assignee,String category) throws Exception;
	/**
	 * 使用角色（groupId）查询可以领取的任务
	 * @param groupId  角色ID	 必须
	 * @param category 系统类别  非必须
	 * @return
	 * @throws Exception
	 */
	public List<TaskModel> findClaimTask(String groupId,String category) throws Exception;
	
	/**
	 * 找到当前人可办理和可领取的任务
	 * @param assignee
	 * @param groupId
	 * @param category
	 * @return
	 * @throws Exception
	 */
	public List<TaskModel> findAssigneedOrClaimTask(String assignee,String groupId,String category) throws Exception;
	
	/**
	 * 任务办理人领取任务
	 * @param taskId   任务ID
	 * @param assignee 领取人
	 */
	public void claimTask(String taskId,String assignee);
	
	/**
	 * 任务办理人完成任务
	 * 状态：正在使用的
	 * 描述：对外提供
     */
	public void completeTask(WorkFlowModel workFlowModel) throws Exception;
	
	/**
	 * 任务委派
	 * @param taskId		   任务ID
	 * @param delegateUserId 被委派人ID
	 */
	public void delegateTask(String taskId,String delegateUserId);
	/**
	 * 
	* @Title: completeTaskByProcInstId  
	* @Description: 通过流程ID和任务处理人完成任务
	* @param
	* @return  
	* @throws
	 */
	public void completeTaskByProcInstIdAndAssignee(String procInstId,Map<String, Object> variables) throws Exception;
	/**
	 * 
	* @Title: completeTaskByProcInstId  
	* @Description: 通过流程ID和任务处理角色完成任务
	* @param
	* @return  
	* @throws
	 */
	public void completeTaskByProcInstIdAndRole(String procInstId,Map<String, Object> variables) throws Exception;
	/**
	 * 根据流程实例ID和任务处理人取出任务
	 * @param procInstId
	 * @param assignee
	 * @return
	 */
	public Task findTaskByProcInstIdAndAssignee(String procInstId,String assignee);
	
	/**
	 * 根据当前任务ID获取下一节点任务信息
	 * @param processDefinitionId   当前流程实例ID
	 * @param activitiId   当前任务ID
	 * @param elString 分支条件el表达式	
	 */
	public TaskDefinition nextTaskDefinition(String processInstanceId,String activitiId,String elString);
}
