/*
 * Copyright (c) 2016, FPX and/or its affiliates. All rights reserved.
 * Use, Copy is subject to authorized license.
*/
package com.qmth.cqb.flow.serviceImpl;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipInputStream;

import org.activiti.engine.HistoryService;
import org.activiti.engine.IdentityService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.history.HistoricProcessInstance;
import org.activiti.engine.repository.Deployment;
import org.activiti.engine.repository.ProcessDefinition;
import org.activiti.engine.repository.ProcessDefinitionQuery;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.DefaultResourceLoader;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.qmth.cqb.flow.model.ProcessModel;
import com.qmth.cqb.flow.model.WorkFlowModel;
import com.qmth.cqb.flow.service.ComProcessService;




/**
 * @Title: 流程相关service
 * @Description:TODO
 * @Company:4px
 * @author chenken
 * @date 2016年6月24日
 * @since v1.0.0
 */
@Service("comProcessService")
public class ComProcessServiceImpl implements ComProcessService{
	 protected Logger logger = LoggerFactory.getLogger(getClass());
	 
	 @Autowired
     protected RuntimeService runtimeService;

     @Autowired
     protected RepositoryService repositoryService;

     @Autowired
     protected HistoryService historyService;
     
     @Autowired
     protected IdentityService identityService;
	/**
	 * 发布流程定义
	 */
	@Override
	public void deployProcess(String processPath,String processName) throws Exception {
		ResourceLoader resourceLoader = new DefaultResourceLoader();
		String classpathResourceUrl = "classpath:/diagrams/"+processPath;
        logger.debug("read workflow from: {}", classpathResourceUrl);
        Resource resource = resourceLoader.getResource(classpathResourceUrl);
        InputStream inputStream = resource.getInputStream();
        if (inputStream == null) {
            logger.warn("ignore deploy workflow module: {}", classpathResourceUrl);
        } else {
            logger.debug("finded workflow module: {}, deploy it!", classpathResourceUrl);
            String fileSuffix = processPath.substring(processPath.lastIndexOf(".")+1,processPath.length());
            //Deployment deployment = null;
            if("ZIP".equals(fileSuffix.toUpperCase())){
            	 ZipInputStream zis = new ZipInputStream(inputStream);
                 repositoryService.createDeployment().addZipInputStream(zis).name(processName).deploy();
            }else{
            	 repositoryService.createDeployment().addInputStream(processPath, inputStream).name(processName).deploy();
            }
            /*if(deployment!=null){
	            // export diagram
	            List<ProcessDefinition> list = repositoryService.createProcessDefinitionQuery().deploymentId(deployment.getId()).list();
	            for (ProcessDefinition processDefinition : list) {
	                WorkflowUtils.exportDiagramToFile(repositoryService, processDefinition, exportDir);
	            }
            }*/
        }
	}
	
	/**
	 * 发布流程定义
	 */
	@Override
	public void deployProcessByInputStream(InputStream inputStream,String processName) throws Exception {
        if (inputStream == null) {
            logger.warn("流程资源为空!");
        } else {
            logger.debug("找到流程资源，开始部署!");
            String fileSuffix = processName.substring(processName.lastIndexOf(".")+1,processName.length());
            Deployment deployment = null;
            if("ZIP".equals(fileSuffix.toUpperCase())){
            	 ZipInputStream zis = new ZipInputStream(inputStream);
                 deployment = repositoryService.createDeployment().addZipInputStream(zis).deploy();
            }else{
            	 deployment = repositoryService.createDeployment().addInputStream(processName, inputStream).deploy();
            }
        }
	}
	
	/**
     * 根据流程实例ID查询流程定义对象{@link ProcessDefinition}
     *
     * @param processInstanceId 流程实例ID
     * @return 流程定义对象{@link ProcessDefinition}
     */
    public ProcessDefinition findProcessDefinitionByPid(String processInstanceId) {
        HistoricProcessInstance historicProcessInstance = historyService.createHistoricProcessInstanceQuery().processInstanceId(processInstanceId).singleResult();
        String processDefinitionId = historicProcessInstance.getProcessDefinitionId();
        ProcessDefinition processDefinition = findProcessDefinition(processDefinitionId);
        return processDefinition;
    }
    
	/**
     * 根据流程定义ID查询流程定义对象{@link ProcessDefinition}
     *
     * @param processDefinitionId 流程定义对象ID
     * @return 流程定义对象{@link ProcessDefinition}
     */
    public ProcessDefinition findProcessDefinition(String processDefinitionId) {
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().processDefinitionId(processDefinitionId).singleResult();
        return processDefinition;
    }
    
	/**
	 * 启动流程
	 */
	@Override
	public String tranStartProcess(String processDefinitionKey,WorkFlowModel workFlowModel) throws Exception {
		Assert.hasText(processDefinitionKey, "流程定义Key不能为空");
		Assert.hasText(workFlowModel.getStarter(), "流程发起人不能为空");
		 // 用来设置启动流程的人员ID，引擎会自动把用户ID保存到activiti:initiator
        identityService.setAuthenticatedUserId(workFlowModel.getStarter());
        Map<String,Object> variables = workFlowModel.getVariables();
        if(StringUtils.isBlank(workFlowModel.getNextOperator())){
			variables.put("operator",null);
		}else{
			variables.put("operator",workFlowModel.getNextOperator());
		}
        String processInstanceId = runtimeService.startProcessInstanceByKey(processDefinitionKey,variables).getProcessInstanceId();
		return processInstanceId;
	}

	/* (non-Javadoc)
	 * @see com.fpx.workflow.activiti.service.ComProcessService#getProcessDefinitionList()
	 */
	@Override
	public List<ProcessModel> getProcessDefinitionList() {
		List<ProcessModel> processModelList = new ArrayList<ProcessModel>();
		ProcessDefinitionQuery processDefinitionQuery = repositoryService.createProcessDefinitionQuery().orderByDeploymentId().desc();  
		List<ProcessDefinition> processDefinitionList = processDefinitionQuery.list();  
		for (ProcessDefinition processDefinition : processDefinitionList) {  
	        String deploymentId = processDefinition.getDeploymentId();  
	        Deployment deployment = repositoryService.createDeploymentQuery().deploymentId(deploymentId).singleResult();
	        processModelList.add(new ProcessModel(processDefinition, deployment));  
	    } 
		return processModelList;
	}

	/* (non-Javadoc)
	 * @see com.fpx.workflow.activiti.service.ComProcessService#deleteProcessDefinition()
	 */
	@Override
	public void deleteProcessDefinition(String deploymentId) {
		//true 表示把流程相关的数据删除
		repositoryService.deleteDeployment(deploymentId, true);
	}

	@Override
	public InputStream getResourceByProcessDefinitionId(String processDefinitionId,String resourceType){
		//流程定义对象
         ProcessDefinition processDefinition = repositoryService
        		.createProcessDefinitionQuery()
        		.processDefinitionId(processDefinitionId)
                .singleResult();
		 String resourceName = "";
         if (resourceType.equals("image")) {
            resourceName = processDefinition.getDiagramResourceName();
         } else if (resourceType.equals("xml")) {
            resourceName = processDefinition.getResourceName();
         }
	     return repositoryService.getResourceAsStream(processDefinition.getDeploymentId(), resourceName);
	}

}
