package com.qmth.cqb.paper.service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;

import com.qmth.cqb.paper.dao.ExtractConfigRepo;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.model.ExamPaper;
import com.qmth.cqb.paper.model.ExtractConfig;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.question.dao.QuesRepo;
import com.qmth.cqb.question.model.QuesOption;
import com.qmth.cqb.question.model.Question;

/**
 * 
 * @author  	chenken
 * @date    	2017年4月14日 下午6:07:55
 * @company 	QMTH
 * @description ExtractConfigServiceImpl.java
 */
@Service("extractConfigService")
public class ExtractConfigServiceImpl implements ExtractConfigService {
	
	@Autowired
    private ExtractConfigRepo extractConfigRepo;
	
	@Autowired
	private PaperDetailRepo paperDetailRepo;

    @Autowired
    private PaperDetailUnitRepo paperDetailUnitRepo;
    
    @Autowired
    private PaperDetailUnitService paperDetailUnitService;
    
    @Autowired
    private PaperService paperService;
    
    @Autowired
    private PaperRepo paperRepo;
    
    @Autowired
    private QuesRepo quesRepo;

    @Autowired
    private ExamPaperService examPaperService;
    
	@Override
	public ExtractConfig findConfig(ExtractConfig condition) {
		if(condition.getExamId()==null){
			return null;
		}
		if(StringUtils.isBlank(condition.getCourseCode())){
			return null;
		}
		ExtractConfig tempConfig =  extractConfigRepo.findOne(Example.of(condition));
        return tempConfig;
	}

	@Override
	public void saveExtractConfig(ExtractConfig extractConfig) {
		if(extractConfig.getScrambling_the_question_order()==null){
			extractConfig.setScrambling_the_question_order((short) 0);
		}
		if(extractConfig.getScrambling_the_option_order()==null){
			extractConfig.setScrambling_the_option_order((short) 0);
		}
		List<ExamPaper> examPapers = extractConfig.getExamPaperList();
		for(int i=0;i<examPapers.size();i++){
			ExamPaper examPaper = examPapers.get(i);
			Paper paper = examPaper.getPaper();
			paper = paperRepo.findOne(paper.getId());
			examPaper.setPaper(paper);
			examPaperService.saveExamPaper(examPaper);//保存
		}
		extractConfigRepo.save(extractConfig);
	}

	@Override
	public ExtractConfig findConfigById(String id) {
		if(StringUtils.isBlank(id)){
			return null;
		}
		return extractConfigRepo.findOne(id);
	}
	
	@Override
	public Map<String, String> makePaperByConfig(String extractConfigId) {
		Map<String, String> finishedPaperIdMap = new HashMap<String, String>();
		//取出调卷规则
		ExtractConfig extractConfig = this.findConfigById(extractConfigId);
		if(extractConfig==null){
			throw new RuntimeException("调卷规则不存在");
		}
		//获得抽取好的试卷
		Map<String,Paper> paperMap = this.getExamPaperByProbability(extractConfig.getExamPaperList());
		if(paperMap.isEmpty()){
			throw new RuntimeException("抽取试卷失败");
		}
        for (Map.Entry<String, Paper> entry : paperMap.entrySet()) {
        	String key = entry.getKey();
        	//根据原有试卷重新组卷得到新试卷
        	Paper newPaper = this.recombinationPaper(entry.getValue(), 
							        				 extractConfig.getScrambling_the_question_order(), 
							        				 extractConfig.getScrambling_the_option_order());
        	finishedPaperIdMap.put(key, newPaper.getId());
        }
        /**
         * 设置map，并保存调卷规则
         * 如果已经设置过了，就不再设置
         */
        if(extractConfig.getIfFinish()==null||extractConfig.getIfFinish()==0){
        	extractConfig.setFinishedPaperIdMap(finishedPaperIdMap);
        	extractConfig.setIfFinish((short)1);
        	extractConfigRepo.save(extractConfig);
        }
        return finishedPaperIdMap;
	}

	/**
	 * 重组试卷，生成新的试卷
	 * @param paper					选中的试卷
	 * @param upSetQuestionOrder	小题乱序		1:乱序  0：不乱序
	 * @param upSetOptionOrder		选项乱序		1:乱序  0：不乱序
	 * @return
	 */
	public Paper recombinationPaper(Paper paper,int upSetQuestionOrder,int upSetOptionOrder){
		List<PaperDetail> paperDetails = paperDetailRepo.findByPaper(paper);
		paper.setId(null);
		Paper newPaper = paperRepo.save(paper);//重新保存成新的paper
		for(int i = 0;i<paperDetails.size();i++){
			PaperDetail paperDetail = paperDetails.get(i);
			List<PaperDetailUnit> paperDetailUnits = paperDetailUnitService.getUnitsByPaperDetail(paperDetail);
			paperDetail.setPaper(newPaper);//关联新Paper
			paperDetail.setId(null);
			PaperDetail newPaperDetail = paperDetailRepo.save(paperDetail);//保存新的paperDetail
			//小题乱序
			if(upSetQuestionOrder==1){
				Collections.shuffle(paperDetailUnits);//打乱小题List
			}
			for(int j = 0;j<paperDetailUnits.size();j++){
				//重新设置保存PaperDetailUnit
				PaperDetailUnit paperDetailUnit = paperDetailUnits.get(j);
				paperDetailUnit.setPaper(newPaper);				//关联新Paper
				paperDetailUnit.setPaperDetail(newPaperDetail); //关联新paperDetail
				paperDetailUnit.setNumber(j+1);					//重新设置序号
				reSavePaperDetailUtilAndQuestion(paperDetailUnit,upSetOptionOrder);
			}
			
		}
		return newPaper;
	}
	
	/**
	 * 每个试卷类型取出一套试卷
	 * {
	 * 	 A:Paper,
	 *   B:Paper
	 * }
	 * A是试卷类型   Paper是A类型下选定的试卷
	 * @param examPaperList		
	 * @return
	 */
	private Map<String,Paper> getExamPaperByProbability(List<ExamPaper> examPaperList){
		Map<String,Paper> paperByTypeMap = new HashMap<String,Paper>();
		if(examPaperList == null || examPaperList.size() == 0) {
       	 	return null;  
        }
		Map<String,List<ExamPaper>> examPaperMap = new HashMap<String,List<ExamPaper>>();
		for(int i = 0;i<examPaperList.size();i++){
			ExamPaper examPaper = examPaperList.get(i);
			if(!examPaperMap.containsKey(examPaper.getGroupCode())){
				if(examPaper.getPaper()!=null){
					List<ExamPaper> epList = new ArrayList<ExamPaper>();
					epList.add(examPaper);
					examPaperMap.put(examPaper.getGroupCode(), epList);
				}
			}else{
				if(examPaper.getPaper()!=null){
					List<ExamPaper> epList = examPaperMap.get(examPaper.getGroupCode());
					epList.add(examPaper);
				}
			}
		}
		if(examPaperMap!=null){
			Set<String> keys = examPaperMap.keySet();
			Iterator<String> it = keys.iterator();
			while (it.hasNext()) {
				String key = it.next();
				Paper paper = this.getPaperByProbability(examPaperMap.get(key));
				//不能用原来的paper对象，否则examPaperList中的paper对象会被覆盖
				Paper newPaper =paperRepo.findOne(paper.getId());
				paperByTypeMap.put(key,newPaper);
			}
		}
        return paperByTypeMap;
	}
	
	/**
	 * 根据设定几率取出一套试卷
	 * @param examPaperList
	 * @return
	 */
	private Paper getPaperByProbability(List<ExamPaper> examPaperList){
		int sum = 0;  
        for (int i = 0;i<examPaperList.size();i++) {  
             sum += examPaperList.get(i).getWeight();  
        }
        // 从1开始  
        Integer rand = new Random().nextInt(sum) + 1;  
        for (int i = 0;i<examPaperList.size();i++) {
            rand -= examPaperList.get(i).getWeight();
            // 选中
            if (rand <= 0) {
                return examPaperList.get(i).getPaper();
            }
        } 
        return null;
	}
	
	/**
	 * 重新设置并保存paperDetailUnit和question
	 * @param paperDetailUnit
	 * @param upSetOptionOrder
	 */
	private void reSavePaperDetailUtilAndQuestion(PaperDetailUnit paperDetailUnit,Integer upSetOptionOrder){
		Question question = paperDetailUnit.getQuestion();
		//选项乱序
		if(upSetOptionOrder==1){
			//单选或多选
			if(question.getQuestionType()==QuesStructType.SINGLE_ANSWER_QUESTION||
			   question.getQuestionType()==QuesStructType.MULTIPLE_ANSWER_QUESTION){
				List<String> numberList = new ArrayList<String>();
				List<QuesOption> options = question.getQuesOptions();
				for(int k = 0;k<options.size();k++){
					QuesOption quesOption = options.get(k);
					numberList.add(quesOption.getNumber());
				}
				Collections.shuffle(numberList);				 	  //打乱number顺序
				paperDetailUnit.setOptionOrder(StringUtils.join(numberList.toArray(),","));//设置option顺序
			}
			//套题，套题下小题不乱序，选择题选项乱序
			if(question.getQuestionType()==QuesStructType.NESTED_ANSWER_QUESTION){
				List<Question> subQuestions = question.getSubQuestions();
				StringBuffer optionOrder = new StringBuffer();
				for(int m=0;m<subQuestions.size();m++){
					Question subQuestion = subQuestions.get(m);
					if(subQuestion.getQuestionType()==QuesStructType.SINGLE_ANSWER_QUESTION||
					   subQuestion.getQuestionType()==QuesStructType.MULTIPLE_ANSWER_QUESTION){
						List<String> numberList = new ArrayList<String>();
						List<QuesOption> options = subQuestion.getQuesOptions();
						for(int n = 0;n<options.size();n++){
							QuesOption quesOption = options.get(n);
							numberList.add(quesOption.getNumber());
						}
						Collections.shuffle(numberList);				 	  //打乱number顺序
						optionOrder.append(StringUtils.join(numberList.toArray(),",")).append(";");
					}
				}
				paperDetailUnit.setOptionOrder(optionOrder.toString());		 //设置option顺序
			}
		}
		paperDetailUnit.setId(null);
		paperDetailUnitRepo.save(paperDetailUnit);//保存新的paperDetailUnit
	}
}
