package com.qmth.cqb.paper.web;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.qmth.cqb.flow.model.DataMapModel;
import com.qmth.cqb.flow.model.WorkFlowModel;
import com.qmth.cqb.flow.service.ComProcessService;
import com.qmth.cqb.flow.service.DataMapModelService;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.service.ImportPaperService;
import com.qmth.cqb.paper.service.PaperService;

import cn.com.qmth.examcloud.common.uac.entity.AccessUser;
import io.swagger.annotations.ApiOperation;

/**
 * Created by songyue on 16/12/28.
 */
@RestController
@RequestMapping("${api_cqb}/")
public class ImportPaperController {

    protected static final Logger log = LoggerFactory.getLogger(ImportPaperController.class);

    @Autowired
    ImportPaperService importPaperService;

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    PaperService paperService;
    
    @Autowired
    ComProcessService comProcessService;
    
    @Autowired
    DataMapModelService dataMapModelService;
    /**
     * 导入试卷
     * 
     * @param paperName
     * @param file
     * @return
     */
    @ApiOperation(value = "导入试卷", notes = "导入试卷")
    @PostMapping(value = "/importPaper")
    public ResponseEntity importPaper(HttpServletRequest request, @RequestParam String paperName,
            @RequestParam String courseNo, @RequestParam String courseName,
            @RequestParam("file") CommonsMultipartFile file) {
        AccessUser user = (AccessUser) request.getAttribute("accessUser");
        log.info("导入开始");
        File tempFile = null;
        String returnStr = "";
        Map map = new HashMap();
        try {
            tempFile = importPaperService.getUploadFile(file);
            returnStr  = importPaperService.ImportPaper(paperName, courseNo, courseName, user, tempFile);
            if (returnStr.equals("success")) {
                return new ResponseEntity(returnStr, HttpStatus.OK);
            } else {
                map.put("errorMsg", returnStr);
                log.info("导入异常:"+returnStr);
                return new ResponseEntity(map, HttpStatus.INTERNAL_SERVER_ERROR);
            }
        } catch (Exception e) {
            e.printStackTrace();
            map.put("errorMsg", e.getMessage());
            log.info("导入异常:"+e.getMessage());
            return new ResponseEntity(map, HttpStatus.INTERNAL_SERVER_ERROR);
        }finally {
            log.info("导入结束");
        }
    }

    /**
     * 新增试卷
     * 
     * @param courseNo
     * @param paperName
     * @return
     */
    @ApiOperation(value = "保存导入类型空白试卷", notes = "保存导入类型空白试卷")
    @PostMapping(value = "/importPaper/saveBlankPaper/{courseNo}/{courseName}/{paperName}")
    public ResponseEntity saveBlankPaper(HttpServletRequest request, @PathVariable String courseNo,
            @PathVariable String courseName, @PathVariable String paperName) {
        AccessUser user = (AccessUser) request.getAttribute("accessUser");
        Map<String, Object> returnMap = importPaperService.saveBlankPaper(courseNo, courseName, paperName, user);
        if (returnMap.get("msg").equals("success")) {
            return new ResponseEntity(returnMap, HttpStatus.OK);
        } else {
            return new ResponseEntity(returnMap, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

}
