package com.qmth.cqb.paper.web;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.cqb.paper.dao.PaperStructRepo;
import com.qmth.cqb.paper.model.PaperStruct;
import com.qmth.cqb.paper.model.PaperStructSearchInfo;
import com.qmth.cqb.paper.service.PaperStructService;

import cn.com.qmth.examcloud.common.uac.entity.AccessUser;
import io.swagger.annotations.ApiOperation;

/**
 * Created by songyue on 16/12/28.
 */
@RestController
@RequestMapping("${api_cqb}/")
public class PaperStructController {

    @Autowired
    PaperStructService paperStructService;

    @Autowired
    PaperStructRepo paperStructRepo;

    /**
     * 获取所有试卷结构
     * 
     * @param
     * @return
     */
    @ApiOperation(value = "获取试卷结构带分页", notes = "获取试卷结构带分页")
    @GetMapping(value = "/paperStruct/{curPage}/{pageSize}")
    public ResponseEntity getPaperStructs(HttpServletRequest request, @ModelAttribute PaperStructSearchInfo searchInfo,
            @PathVariable int curPage, @PathVariable int pageSize) {
        AccessUser user = (AccessUser) request.getAttribute("accessUser");
        return new ResponseEntity(paperStructService.getPaperStructs(searchInfo, curPage, pageSize, user),
                HttpStatus.OK);
    }

    /**
     * 获取所有试卷结构
     * 
     * @param
     * @return
     */
    @ApiOperation(value = "获取试卷结构", notes = "获取试卷结构")
    @GetMapping(value = "/paperStruct")
    public ResponseEntity findAll() {
        return new ResponseEntity(paperStructRepo.findAll(), HttpStatus.OK);
    }

    /**
     * 根据id获取试卷结构
     * 
     * @param
     * @return
     */
    @ApiOperation(value = "获取试卷结构", notes = "获取试卷结构")
    @GetMapping(value = "/paperStruct/{id}")
    public ResponseEntity getPaperStructById(@PathVariable String id) {
        return new ResponseEntity(paperStructRepo.findOne(id), HttpStatus.OK);
    }

    /**
     * 更新试卷结构
     * 
     * @param
     * @return
     */
    @ApiOperation(value = "更新试卷结构", notes = "更新试卷结构")
    @PutMapping(value = "/paperStruct")
    public ResponseEntity updatePaperStruct(HttpServletRequest request, @RequestBody PaperStruct ps) {
        AccessUser user = (AccessUser) request.getAttribute("accessUser");
        PaperStruct paperStruct = paperStructService.save(ps, user);
        return new ResponseEntity(paperStruct, HttpStatus.OK);
    }

    /**
     * 新增试卷结构
     * 
     * @param ps
     * @return
     */
    @ApiOperation(value = "新增试卷结构", notes = "新增试卷结构")
    @PostMapping(value = "/paperStruct")
    public ResponseEntity addPaperStruct(HttpServletRequest request, @RequestBody PaperStruct ps) {
        AccessUser user = (AccessUser) request.getAttribute("accessUser");
        PaperStruct paperStructTemp = paperStructService.checkNameUnique(ps.getName(), user.getOrgId().toString());
        if (paperStructTemp != null) {
            return new ResponseEntity("试卷结构名称重复，请重新命名！", HttpStatus.INTERNAL_SERVER_ERROR);
        } else {
            PaperStruct paperStruct = paperStructService.save(ps, user);
            return new ResponseEntity(paperStruct, HttpStatus.OK);
        }
    }

    /**
     * 删除试卷结构
     * 
     * @param ids
     * @return
     */
    @ApiOperation(value = "删除试卷结构", notes = "删除试卷结构")
    @DeleteMapping(value = "/paperStruct/{ids}")
    public ResponseEntity removePaperStruct(@PathVariable String ids) {
        List<String> paperList = Stream.of(ids.split(",")).collect(Collectors.toList());
        paperStructRepo.delete(paperStructRepo.findAll(paperList));
        return new ResponseEntity(HttpStatus.OK);
    }
}
