package com.qmth.cqb;

import java.util.List;

import javax.annotation.Resource;

import org.activiti.engine.HistoryService;
import org.activiti.engine.IdentityService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.history.HistoricActivityInstance;
import org.activiti.engine.history.HistoricVariableInstance;
import org.activiti.engine.identity.Group;
import org.activiti.engine.identity.User;
import org.activiti.engine.repository.ProcessDefinition;
import org.activiti.engine.task.Task;
import org.activiti.engine.test.Deployment;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import com.qmth.cqb.flow.model.WorkFlowModel;
import com.qmth.cqb.flow.service.ComProcessService;
import com.qmth.cqb.flow.service.ComTaskService;





@RunWith(SpringRunner.class)
@SpringBootTest
public class FlowTest{
	
	@Resource
	public ComProcessService comProcessService;
	@Resource
	private TaskService taskService;
	@Resource
	private HistoryService historyService;
	@Resource
	private IdentityService identityService;
	@Resource
	private ComTaskService comTaskService;
	@Resource
	private RuntimeService runtimeService;
	@Resource
    protected RepositoryService repositoryService;
	
	@Test
	@Deployment
	public void testDeploy() throws Exception{
		 //流程文件名不能使用中文
		 comProcessService.deployProcess("importPaperAduit.bpmn", "导入试卷流程");
	}
	 @Test 
	public void testQueryAllTask(){
		List<Task> taskList = taskService.createTaskQuery().list();
		for(Task task:taskList){
			String processDefinitionId = task.getProcessDefinitionId();
			ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().processDefinitionId(processDefinitionId).singleResult();
			Assert.assertNotNull(processDefinition);
		}
		System.out.println(taskList);
	}
	
    /**查询我的个人任务,没有执行结果*/  
    @Test  
    public void findPersonalTaskList() {  
        // 任务办理人  
        String assignee = "chenken";  
        List<Task> list = taskService//  
                .createTaskQuery()//  
                .taskAssignee(assignee)// 个人任务的查询  
                .list();  
        if (list != null && list.size() > 0) {  
            for (Task task : list) {  
                System.out.println("任务ID：" + task.getId());  
                System.out.println("任务的办理人：" + task.getAssignee());  
                System.out.println("任务名称：" + task.getName());  
                System.out.println("任务的创建时间：" + task.getCreateTime());  
                System.out.println("流程实例ID：" + task.getProcessInstanceId());  
                System.out.println("#######################################");  
            }  
        }  
    }  
    /**查询组任务*/
	@Test
	public void findGroupTaskList(){
		 // 任务办理人  
        String candidateUser = "admin";  
		List<Task> list = taskService  
                .createTaskQuery() 
                .taskCandidateUser(candidateUser)// 参与者，组任务查询  
                .list(); 
		if (list != null && list.size() > 0) {  
            for (Task task : list) {  
                System.out.println("任务ID：" + task.getId());  
                System.out.println("任务的办理人：" + task.getAssignee());  
                System.out.println("任务名称：" + task.getName());  
                System.out.println("任务的创建时间：" + task.getCreateTime());  
                System.out.println("流程实例ID：" + task.getProcessInstanceId());  
                System.out.println("#######################################");  
            }  
        }
	}
	
	 /**完成任务*/  
    @Test  
    public void completeTask() {  
        // 任务ID  
        String taskId = "742506";  
        taskService.complete(taskId);  
        System.out.println("完成任务：" + taskId);  
    } 
    /**
     * 查询流程变量
     */
    @Test 
    public void getRuTaskVariable(){
    	System.out.println(taskService.getVariable("87511", "businessStr"));
    }
    
    @Test 
    public void getHistoryActivity(){
    	String processInstanceId = "100001";
    	List<HistoricActivityInstance> historicActivityInstances = 
    	historyService.createHistoricActivityInstanceQuery().processInstanceId(processInstanceId).list();
    	List<HistoricVariableInstance> historicVariableInstances =
    			historyService.createHistoricVariableInstanceQuery().processInstanceId(processInstanceId).list();
    }
    /**
     * 用户与组之间的关系
     */
    @Test
    public void testUserAndGroupMemership(){
    	/*创建组
    	Group group = identityService.newGroup("aduiter");
    	group.setName("B端客户审核员");
    	identityService.saveGroup(group);*/
    	//创建用户
    	User user = identityService.newUser("ck");
    	user.setFirstName("chen");
    	user.setLastName("ken");
    	user.setEmail("1234567@qq.com");
    	identityService.saveUser(user);
    	//把用户ck加入到组aduiter中
    	Group group = identityService.createGroupQuery().groupId("aduiter").singleResult();
    	identityService.createMembership(user.getId(), group.getId());
    	//查询组aduiter 下的用户
    	List<User> users = identityService.createUserQuery().memberOfGroup(group.getId()).list();
    	Assert.assertEquals("ck",users.get(0).getId());
    	//查询ck所在的组
    	Group groupContainsCk = identityService.createGroupQuery().groupMember(user.getId()).singleResult();
    	Assert.assertNotNull(groupContainsCk);
    	Assert.assertEquals("aduiter", groupContainsCk.getId());
    }
    @Test
    public void testGroup(){
    	List<Group> groupList = identityService.createGroupQuery().groupId("aduiter").list();
    	Assert.assertEquals(1,groupList.size());
    }
    @Test
    public void deleteMembership(){
    	identityService.deleteMembership("ck", "aduiter");
    	identityService.deleteGroup("aduiter");
    	identityService.deleteUser("ck");
    }
    
    @Test
    public void getTaskByGroupId(){
    	//根据角色ID查询用户任务
    	List<Task> taskList = taskService.createTaskQuery().taskCandidateGroup("aduiter").active().list();
    	//lihao领取任务
    	for(Task task:taskList){
    		taskService.claim(task.getId(), "lihao");
    	}
    	/*
    	Task task = taskService.createTaskQuery().taskAssignee("lihao").active().singleResult();
    	
    	System.out.println(task);
    	Map<String,Object> valiables = new HashMap<String,Object>();
    	valiables.put("approval", "backward-completed");
    	taskService.complete(task.getId(),valiables);
    	task = taskService.createTaskQuery().taskAssignee("lihao").active().singleResult();
    	
    	System.out.println(task);*/
    }
    
    @Test
    public void testStartTask() throws Exception{
    	WorkFlowModel workFlowModel = new WorkFlowModel();
    	workFlowModel.setStarter("liuyun");
    	workFlowModel.setNextOperator("chenken");
    	String  processInstanceId = comProcessService.tranStartProcess("",workFlowModel);
    	System.out.println(processInstanceId);
    }
    
    @Test
    public void testCompleteTask() throws Exception{
    	/*//String category = EnumSystem.CRM.getName();
    	String roleId = "group1";
    	String assignee = "chenken";
    	//找到可领取的任务
    	List<TaskModel> claimTasks = comTaskService.findClaimTask(roleId,"ABCD");
    	for(TaskModel task:claimTasks){
    		//领取任务
    		comTaskService.claimTask(task.getTaskId(),assignee);
    	}
    	//找到可完成的任务
    	List<TaskModel> tasks = comTaskService.findPersonalTaskList(assignee,category);
    	*//**
    	 * 委派
    	 * comTaskService.delegateTask(tasks.get(0).getId(), "xxyy");
    	 *//*
    	for(TaskModel task:tasks){
    		//完成任务
    		WorkFlowModel workFlowModel = new WorkFlowModel();
        	workFlowModel.setNodeServiceName("completeUserTaskService");
        	workFlowModel.setTaskId(task.getTaskId());
        	workFlowModel.setAssignee(assignee);
        	workFlowModel.setCondition("1");
        	workFlowModel.setNextOperator("liuyun");
        	CompleteUserTaskService workFlowService = WorkflowUtils.getCurrentTaskService(workFlowModel.getNodeServiceName());
        	workFlowService.completeUserTask(workFlowModel);
    	}*/
    }
    
}
